--//////////Peripheral_Expansion_Top /////////////////////////////////--
-- ***********************************************************************
-- FileName: Peripheral_Expansion_Top.vhd
-- FPGA: Lattice MachXO2 1200ZE-1TG144C
-- IDE: Lattice Diamond ver. 1.4.2.105
--
-- HDL IS PROVIDED "AS IS." DIGI-KEY EXPRESSLY DISCLAIMS ANY
-- WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
-- PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL DIGI-KEY
-- BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
-- DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
-- PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
-- BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
-- ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
-- DIGI-KEY ALSO DISCLAIMS ANY LIABILITY FOR PATENT OR COPYRIGHT
-- INFRINGEMENT.
--
-- Version History
-- Version 1.0 15/06/2012 Tony Storey
-- Initial Public Release




library IEEE;
use IEEE.std_logic_1164.all;
library MACHXO2;
use MACHXO2.components.all;

entity Peripheral_Expansion_Top is
      Port (      
				Gray_in 		: In std_logic_vector(3 downto 0);
				nreset 			: In    std_logic;
				spi_clk 			: Inout std_logic;
				spi_mosi 	: Inout std_logic;
				spi_scsn 		: In std_logic;
				a_in 				: In std_logic;
				b_in				: In std_logic;
				tct_sw			: In std_logic_vector(7 downto 0);
				busy_out		: Out std_logic;
				an 					: Out   std_logic_vector (3 downto 0);
                sseg 				: Out   std_logic_vector (7 downto 0);
				LED_out 		: Out std_logic_vector(7 downto 0);
				spi_miso 	: Inout std_logic;
				spi_irq 			: Out std_logic
			);
end Peripheral_Expansion_Top;

architecture arch of Peripheral_Expansion_Top is


--	signal 		nreset 	: std_logic := 1;
	signal 		gnd 		: std_logic := '0';
	signal 		vcc 		: std_logic := '1';

  
	signal     N_15 		: std_logic;
   
	signal     N_16 		: std_logic;
   
	signal   	bus_in 		: std_logic_vector (31 downto 0);

--	signal     hex3 		: std_logic_vector (3 downto 0);

--	signal     hex2 		: std_logic_vector (3 downto 0);

	--signal     hex1 		: std_logic_vector (3 downto 0);

	--signal     hex0 		: std_logic_vector (3 downto 0);

	signal    sseg0 		: std_logic_vector (7 downto 0);

	signal    sseg1 		: std_logic_vector (7 downto 0);

	signal    sseg2 		: std_logic_vector (7 downto 0);

	signal    sseg3 		: std_logic_vector (7 downto 0);

	signal  	clk  			: std_logic;
	
	-- local wishbone bus signals from wb state machine controller and data mux
	signal w_enable	: std_logic;
	
	signal wb_rst_i		: std_logic;
	
	signal wb_cyc_i 	: std_logic;
	
	signal wb_stb_i 	: std_logic;
	
	signal wb_we_i 	: std_logic;
	
	signal wb_adr_i 	: std_logic_vector (7 downto 0);
	
	signal wb_dat_i 	: std_logic_vector (7 downto 0);
	
	signal wb_dat_o2mux	: std_logic_vector(7 downto 0);
	
	signal wb_dat_o 	: std_logic_vector (7 downto 0);
	
	signal wb_ack_o 	: std_logic;
	
	signal wb_busy	: std_logic;
	
	signal wb_bus_en	: std_logic;
			
	signal reg_address_o 		: std_logic_vector(7 downto 0);
	
	signal tx_address				: std_logic_vector(7 downto 0);
	
	signal tx_reg							: std_logic_vector(7 downto 0);	
	
	signal Decode_out				: std_logic_vector(7 downto 0);
	
	signal tct_sw_wire				: std_logic_vector(7 downto 0);
	
--internal oscillator
   component OSCH
      Generic(
            NOM_FREQ: string := "38.00");
      Port( 
            STDBY    		: in std_logic;
            OSC      			: out std_logic;
            SEDSTDBY	: out std_logic);
   end component;	


   component GrayDecoder
      Port ( 
				Gray_in  			: In   std_logic_vector(3 downto 0);
				Decode_out 	: Out std_logic_vector(3 downto 0)
				);
   end component;

   component disp_mux
      Port (     
				clk 				: In    std_logic;
                in0 			: In    std_logic_vector (7 downto 0);
                in1 			: In    std_logic_vector (7 downto 0);
                in2 			: In    std_logic_vector (7 downto 0);
                in3 			: In    std_logic_vector (7 downto 0);
				n_reset 	: In    std_logic;
                an 				: Out   std_logic_vector (3 downto 0);
                sseg 			: Out   std_logic_vector (7 downto 0) 
				);
   end component;

   component hex_to_sseg
      Port (      
				dp 				: In    std_logic;
                hex 			: In    std_logic_vector (3 downto 0);
                sseg 			: Out   std_logic_vector (7 downto 0) 
				);
   end component;

   component qei
      Port (    
				a_in 							: In    std_logic;
                b_in 							: In    std_logic;
                clk 								: In    std_logic;
				n_reset 					: In    std_logic;
				signed_QEI_reg 	: Out   std_logic_vector(31 downto 0)
			--	signed_vel_reg : Out   std_logic_vector(31 downto 0) 
				);
   end component;

   component dbounce
      Port ( 
				button_in 	: In    std_logic;
				clk 					: In    std_logic;
				nreset 			: In    std_logic;
				DB_out 		: buffer std_logic 
				);
   end component;

	component SPI_Bus
		port (
				wb_clk_i		: in  std_logic; 
				wb_rst_i		: in  std_logic; 
				wb_cyc_i		: in  std_logic; 
				wb_stb_i		: in  std_logic; 
				wb_we_i		: in  std_logic; 
				wb_adr_i		: in  std_logic_vector(7 downto 0); 
				wb_dat_i		: in  std_logic_vector(7 downto 0); 
				wb_dat_o	: out  std_logic_vector(7 downto 0); 
				wb_ack_o	: out  std_logic; 
				spi_clk			: inout  std_logic; 
				spi_miso		: inout  std_logic; 
				spi_mosi		: inout  std_logic; 
				spi_scsn		: in  std_logic; 
				spi_irq			: out  std_logic
				);
	end component;

	component SPI_wbController
		port(
				clk										: in  std_logic;
				nreset								: in  std_logic;
				wb_bus_en					: in  std_logic;
				wb_ack_o						: in  std_logic;
				wb_valid_address 		: in std_logic_vector(7 downto 0);
				wb_valid_data				: in std_logic_vector(7 downto 0);
				wb_valid_data_o			: in std_logic_vector(7 downto 0);
				w_enable						: in  std_logic;												-- inputs 0 for read and 1 for write
				busy						: out  std_logic;
				wb_rst_i				: out  std_logic;
				wb_cyc_i				: out  std_logic; 
				wb_stb_i				: out  std_logic; 
				wb_we_i				: out  std_logic; 
				wb_adr_i				: out std_logic_vector(7 downto 0); 
				wb_dat_o			: out std_logic_vector(7 downto 0);
				wb_dat_i				: out  std_logic_vector(7 downto 0)
				);
	end component;
	
	component SPI_Data_MUX 
	port(
				clk									: in std_logic;
				nreset							: in std_logic;
				wb_busy						: in std_logic;
				reg_address				: in std_logic_vector(7 downto 0);				-- comes from recieve register
				QEI_0							: in std_logic_vector(7 downto 0);				--  from the respective QEI module
				QEI_1							: in std_logic_vector(7 downto 0);
				QEI_2							: in std_logic_vector(7 downto 0);
				QEI_3							: in std_logic_vector(7 downto 0);
				QEI_4							: in std_logic_vector(7 downto 0);
				Gray_code_0				: in std_logic_vector(7 downto 0);				-- from the gray code decoders
				Gray_code_1				: in std_logic_vector(7 downto 0);
				tct_sw							: in std_logic_vector(7 downto 0);				-- from the button inputs
				reg_address_o	: buffer std_logic_vector(7 downto 0);               -- testing to get output from recieve reg
				wb_bus_en	: out std_logic;																	-- output to enable a write on the wb_bus
				w_enable		: out std_logic;																	-- output 0 for read and 1 for write
				tx_reg				: out  std_logic_vector(7 downto 0);						-- data to transmit back to master
				tx_address		: out  std_logic_vector(7 downto 0)							-- address to put the data or pull the data 0x59 for transmit buffer and 0x5B for reciever buffer and 0x5A for status reg
				);
end component;
	
	
		
begin

  
	  
-- used to instantiate on chip osc
   OSCInst0: OSCH
      Generic Map (NOM_FREQ  => "38.00")
      Port Map (STDBY => '0', OSC => clk,  SEDSTDBY => open);	  

-- The debounced inputs are for the byte wide input connected to the active low tactile buttons
  I18 : dbounce
      Port Map ( button_in => tct_sw(0), clk => clk, nreset => nreset,
					DB_out => tct_sw_wire(0) );

  I19 : dbounce
      Port Map ( button_in => tct_sw(1), clk => clk, nreset => nreset,
					DB_out => tct_sw_wire(1)  );
					
  I20 : dbounce
      Port Map ( button_in => tct_sw(2), clk => clk, nreset => nreset,
					DB_out => tct_sw_wire(2)  );

  I21 : dbounce
      Port Map ( button_in => tct_sw(3), clk => clk, nreset => nreset,
					DB_out => tct_sw_wire(3)  );
					
  I22 : dbounce
      Port Map ( button_in => tct_sw(4), clk => clk, nreset => nreset,
					DB_out => tct_sw_wire(4)  );
					
  I23 : dbounce
      Port Map ( button_in => tct_sw(5), clk => clk, nreset => nreset,
					DB_out => tct_sw_wire(5)  );

  I24 : dbounce
      Port Map ( button_in => tct_sw(6), clk => clk, nreset => nreset,
					DB_out => tct_sw_wire(6)  );

   I25 : dbounce
      Port Map ( button_in => tct_sw(7), clk => clk, nreset => nreset,
					DB_out => tct_sw_wire(7)  );

-- The spi data mux is instantiated to control the flow of incoming and outgoing data to the spi controller interfacing the wishbone bus
  I17 : SPI_Data_Mux
	port map( clk => clk, nreset => nreset, wb_busy => wb_busy, reg_address( 7 downto 0) => wb_dat_o2mux(7 downto 0),
						QEI_0 => bus_in(7 downto 0) , QEI_1 => X"00",  QEI_2 => X"00",  QEI_3 => X"00",  QEI_4 => X"00", 
						Gray_code_0 =>  Decode_out(7 downto 0) , Gray_code_1 => X"00", tct_sw(7 downto 0) => tct_sw_wire(7 downto 0), 
						reg_address_o(7 downto 0) => reg_address_o(7 downto 0), wb_bus_en => wb_bus_en, w_enable => w_enable, tx_reg(7 downto 0) => tx_reg(7 downto 0), 
						tx_address(7 downto 0) => tx_address(7 downto 0));
	  
-- The spi bus is instantiated through the EFB within IPExpress
  I15 : SPI_Bus
    port map (wb_clk_i => clk,  wb_rst_i => wb_rst_i,  wb_cyc_i => wb_cyc_i,  wb_stb_i => wb_stb_i, 
						wb_we_i => wb_we_i,  wb_adr_i(7 downto 0) => wb_adr_i(7 downto 0),  
						wb_dat_i(7 downto 0) => wb_dat_i(7 downto 0),  wb_dat_o(7 downto 0) => wb_dat_o(7 downto 0) , 			-- This is the data that comes from the master such as an address to mux to the tx buffer to the wb_data_i
						wb_ack_o => wb_ack_o,  spi_clk => spi_clk,  spi_miso => spi_miso, 
						spi_mosi => spi_mosi,  spi_scsn => spi_scsn,  spi_irq => spi_irq);  
	
	
-- The spi controller is used to control the machxo2 wishbone bus interfacing the spi
  I16 : SPI_wbController
	port map ( clk => clk,  nreset => nreset, wb_bus_en  => wb_bus_en,  wb_ack_o => wb_ack_o, 
						wb_valid_address(7 downto 0) => tx_address(7 downto 0),													
						w_enable => w_enable, busy => wb_busy,							---w_enable,
						wb_valid_data(7 downto 0) => 	tx_reg(7 downto 0),																			--tx_reg(7 downto 0),  
						wb_valid_data_o(7 downto 0) =>  wb_dat_o(7 downto 0),	-----------------------------
						wb_rst_i => wb_rst_i,  wb_cyc_i => wb_cyc_i,  wb_stb_i => wb_stb_i, 
						wb_we_i => wb_we_i,   wb_adr_i(7 downto 0) => wb_adr_i(7 downto 0), 
						wb_dat_o(7 downto 0) => wb_dat_o2mux(7 downto 0),  --------------------------------
						wb_dat_i(7 downto 0) => wb_dat_i(7 downto 0));																												-- This is the data going out to the master from the txbuffer		
	  
	busy_out <= wb_busy;			-- connect wb_busy signal to output pin for testing
	

-- The gray decoder is instantiated to decode the gray to standard hex progression
  I1 : GrayDecoder
      Port Map ( Gray_in(3 downto 0) => Gray_in(3 downto 0),
						Decode_out(3 downto 0) => Decode_out(3 downto 0));
						
						Decode_out(7 downto 4) <= X"0";
						Led_out(3 downto 0) <= not(Decode_out(3 downto 0));
						
						
-- The disp_mux is instantiated to route each nibble to its corrisponding sseg character and multiplex the anodes 
  I2 : disp_mux
      Port Map ( clk=>clk, n_reset=>nreset,
						in0(7 downto 0) => sseg0(7 downto 0),
						in1(7 downto 0) => sseg1(7 downto 0),
						in2(7 downto 0) => sseg2(7 downto 0),
						in3(7 downto 0) => sseg3(7 downto 0), 
						an(3 downto 0) => an(3 downto 0),
						sseg(7 downto 0) => sseg(7 downto 0) );
	
-- The most siginificant byte going to the seven seg display is the address info from the master
   I3 : hex_to_sseg
      Port Map ( dp=>vcc, hex(3 downto 0) => reg_address_o(7 downto 4),												
					sseg(7 downto 0) => sseg3(7 downto 0));
					
   I4 : hex_to_sseg
      Port Map ( dp=>vcc, hex(3 downto 0) =>  reg_address_o(3 downto 0),											
					sseg(7 downto 0) => sseg2(7 downto 0));
					
-- The least significant byte going to the seven seg display is the encoder position				
   I5 : hex_to_sseg
      Port Map ( dp=>vcc, hex(3 downto 0) => bus_in(7 downto 4),
					sseg(7 downto 0) => sseg1(7 downto 0));

  I6 : hex_to_sseg
      Port Map ( dp => vcc, hex(3 downto 0) => bus_in(3 downto 0),		
					sseg(7 downto 0) => sseg0(7 downto 0) );
	
-- The qui keeps track of the encoder postion				
   I14 : qei
      Port Map ( a_in => N_15, b_in => N_16, clk => clk, n_reset => nreset,
					signed_QEI_reg(31 downto 0) => bus_in(31 downto 0) );
	
-- The two debouce instantiations are for the a and b, quadriture encoder channels 
   I12 : dbounce
      Port Map ( button_in => a_in, clk => clk, nreset => nreset,
					DB_out => N_15 );

  I13 : dbounce
      Port Map ( button_in => b_in, clk => clk, nreset => nreset,
					DB_out => N_16 );




end arch;
