--//////////QEI /////////////////////////////////--
-- ***********************************************************************
-- FileName: QEI.vhd
-- FPGA: Lattice MachXO2 1200ZE-1TG144C
-- IDE: Lattice Diamond ver. 1.4.2.105
--
-- HDL IS PROVIDED "AS IS." DIGI-KEY EXPRESSLY DISCLAIMS ANY
-- WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
-- PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL DIGI-KEY
-- BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
-- DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
-- PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
-- BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
-- ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
-- DIGI-KEY ALSO DISCLAIMS ANY LIABILITY FOR PATENT OR COPYRIGHT
-- INFRINGEMENT.
--
-- Version History
-- Version 1.0 15/06/2012 Tony Storey
-- Initial Public Release




library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

-- Part of this module has been commented out because velocity doesn't apply to this design
-- if rotary encoder velocity is desired, uncomment accordingly

entity QEI is
	Generic(
		dt 						: integer 	:= 3325000;		-- 50 ms * 66.5 MHz = 3325000, sets time to find rate dc/dt
		Neg_Cnt 			: integer := 0;			-- option to count negative positions set 1 for true 0 for only positive postitions (0 is smallest reg value)
		N						: integer := 16					-- width of QEI counter
		);
    Port ( 
			clk 							: in  STD_LOGIC;
           n_reset 				: in  STD_LOGIC;
           a_in 						: in  STD_LOGIC;
           b_in 						: in  STD_LOGIC;
           signed_QEI_reg : out  STD_LOGIC_VECTOR (31 downto 0)
  --         signed_vel_reg : out  STD_LOGIC_VECTOR (N-1 downto 0)
			);
end QEI;



architecture arch of QEI is
	-- comment counter and counter control if one doesn't need velocity
	------------------------------------------------------------------------------------
	--signal q_reg, q_next 			: unsigned(N-1 downto 0);					-- regs for counter
	--signal q_reset, q_add			: std_logic;											-- control flags for counter 
	signal a_in_dff							: std_logic;											-- flip flop for for rising edge detection for a quadriture signal
	signal trn_rt, trn_lt					: std_logic;											-- flags for determining direction
	signal trn_cont							: std_logic_vector(1 downto 0);			-- control word for QEI position tracking
	signal a_in_rising_edge		: std_logic;											-- rising edge detection flag
	signal QEI_reg, QEI_next		: std_logic_vector(N-1 downto 0);		-- QEI reg to track position
	--signal QEI_reg_temp				: std_logic_vector(N-1 downto 0);		-- QEI temp reg for calculating dc/dt (velocity)


begin


	process(clk, n_reset)
	begin
		if(n_reset = '0') then
--			q_reg <= (others => '0');
			QEI_reg <= (others => '0');
		elsif rising_edge(clk) then
--			q_reg <= q_next;
			QEI_reg <= QEI_next;
		end if;
	end process;
	
	 --combinational circuit for timer logic
--	q_next <= (others => '0') when q_reset = '1' else		-- reset the counter
--					q_reg + 1 when q_add = '1' else				-- increment count if commanded
--					q_reg;														-- maintain current value if no command

 
	-- circiut to compute velocity dq/dt
	--process(q_reg)
	--begin 
--		 defualt
		--q_reset <= '0';
		--q_add <= '0';
		
		--if(q_reg <= 0) then
			--QEI_reg_temp <= QEI_reg;
			--q_add <= '1';
		--elsif(q_reg < dt) then
			--q_add <= '1';
		--elsif(q_reg >= dt) then
			--signed_vel_reg <= std_logic_vector(signed(QEI_reg) - signed(QEI_reg_temp));
			--q_reset <= '1';
		--end if;
		
	--end process;
	

	-- dff for finding rising edge of a_in signal
	process(a_in, a_in_dff, clk)
	begin
		if rising_edge(clk) then
			a_in_dff <= a_in;
			a_in_rising_edge <= a_in and (a_in xor a_in_dff);
		end if;
	end process;
	

	
	-- sequential process to raise trn_rt and trn_lt flags
	process(a_in_rising_edge, a_in, b_in)
	begin
		trn_rt <= a_in_rising_edge and a_in and not b_in;
		trn_lt <= a_in_rising_edge and a_in and b_in;
	end process;

	trn_cont <= trn_rt & trn_lt;	
	
	-- sequential process to keep track of QEI position
	process(QEI_reg, trn_cont)
	begin
			
			case trn_cont is
				
				when "10" =>
					QEI_next <= std_logic_vector(signed(QEI_reg) + 1);					
				
				when "01" =>
					if(Neg_Cnt = 1) then
						QEI_next <= std_logic_vector(signed(QEI_reg) - 1);				
					elsif(Neg_Cnt = 0) then
						if(signed(QEI_reg) <= 0) then
							QEI_next <= QEI_reg;
						else
							QEI_next <= std_logic_vector(signed(QEI_reg) - 1);
						end if;
					end if;
					
				when others =>
				
					QEI_next <= QEI_reg;
			end case;
 
	end process;

signed_QEI_reg <= "0000000000000000" & QEI_next;








end arch;
