--//////////disp_mux /////////////////////////////////--
-- ***********************************************************************
-- FileName: disp_mux.vhd
-- FPGA: Lattice MachXO2 1200ZE-1TG144C
-- IDE: Lattice Diamond ver. 1.4.2.105
--
-- HDL IS PROVIDED "AS IS." DIGI-KEY EXPRESSLY DISCLAIMS ANY
-- WARRANTY OF ANY KIND, WHETHER EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
-- PARTICULAR PURPOSE, OR NON-INFRINGEMENT. IN NO EVENT SHALL DIGI-KEY
-- BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL
-- DAMAGES, LOST PROFITS OR LOST DATA, HARM TO YOUR EQUIPMENT, COST OF
-- PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
-- BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF),
-- ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.
-- DIGI-KEY ALSO DISCLAIMS ANY LIABILITY FOR PATENT OR COPYRIGHT
-- INFRINGEMENT.
--
-- Version History
-- Version 1.0 15/06/2012 Tony Storey
-- Initial Public Release


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;


entity disp_mux is
    Port ( 
			clk : in  STD_LOGIC;
			n_reset : in  STD_LOGIC;
			in3 : in  STD_LOGIC_VECTOR (7 downto 0);
			in2 : in  STD_LOGIC_VECTOR (7 downto 0);
			in1 : in  STD_LOGIC_VECTOR (7 downto 0);
			in0 : in  STD_LOGIC_VECTOR (7 downto 0);
			an : out  STD_LOGIC_VECTOR (3 downto 0);
			sseg : out  STD_LOGIC_VECTOR (7 downto 0));
end disp_mux;

architecture arch of disp_mux is
	-- refreshing rate around 290 Hz ( 38.00 MHz/2^17)
	constant N: integer := 17;
	signal q_reg, q_next: unsigned(N-1 downto 0);
	signal sel: std_logic_vector(1 downto 0);

   
begin
 




	-- register updates
	process(clk, n_reset)
	begin
		if (n_reset = '0') then
			q_reg <= (others => '0');
		elsif (clk'event and clk = '1') then
			q_reg <= q_next;
		end if;
	end process;
	
	-- next state logic for the counter
	q_next <= q_reg + 1;
	
	-- 2 MSBs of counter to control 4-to-1 multiplexing
	-- and to generate active-low enable signal
	sel <= std_logic_vector(q_reg(N-1 downto N-2));
	process(sel, in0, in1, in2, in3)
	begin
		case sel is
			when "00" =>
				an <= "0001";
				sseg <= in3;
			when "01" =>
				an <= "0010";
				sseg <= in2;
			when "10" =>
				an <= "0100";
				sseg <= in1;
			when others =>
				an <= "1000";
				sseg <= in0;
		end case;
	
	end process;
	


end arch;

